#_____________________________________________________________________________#
#|                                                                           |#
#| This file contains procedures to invoke the bindings for adjusting points |#
#| on polygons.                                                              |#
#| 1. proc Create_Co_Points :                                                |#
#|         When one of the radiobuttons are invoked to carry out an action on|#
#|         the points of a poly, this procedure creates a series of          |#
#|         coordinate points on the selected polygon that can be picked up.  |#
#| 2. proc Move_Poly_Point_BP1 :                                             |#
#|         Creates the binding for <ButtonPress-1> for the "Add point" and   |#
#|         "Move-point" options.                                             |#
#| 3. proc Drag_Poly_Point :                                                 |#
#|         Creates the binding for <B1-Motion> for the "Add point" and "Move-|#
#|         point" options.                                                   |#
#| 4. proc Move_Poly_Point_BR1 :                                             |#
#|         Creates the binding for <ButtonRelease-1> for the "Move point"    |#
#|         option.                                                           |#
#| 5. proc Add_Poly_Point_BR1 :                                              |#
#|         Creates the binding for <ButtonRelease-1> for the "Add point"     |#
#|         option.                                                           |#
#|___________________________________________________________________________|#
#                                                                             #

#=============================================================================#

###############################################################################
# This procedure creates the coordinate points for a selected polygon if      #
# 'move/add/delete point' button is selected.  This is carried out when the   #
# appropriate button is pressed.  The display will be returned to normal on   #
# button-release-1 in the canvas.  If the selected item is not a polygon, then#
# none of the following bindings will occur.  Instead a message is displayed  #
#on the message widget beneath the canvas.                                    #
###############################################################################
proc Create_Co_Points {} {

# Gain access to global "_CHOICE"
    global _CHOICE

# Set these bindings to have no commands.  This removes any rogue bindings
    .can.c1 bind selected <ButtonPress-1> {}
    bind .can.c1 <B1-Motion> {}
    bind .can.c1 <ButtonRelease-1> {}

# Set variable to "type" of selected item
    set type [.can.c1 type selected]

# If the item is a polygon then create the bindings.
# Else set message to inform the user of his options
    if {$type=="polygon" || $type=="line"} {

# If the "Delete_Point" option is chosen, and the selected polygon has only three
# points, display a message telling the user, and return to the default bindings.
        if {$_CHOICE=="DELETE_POINT" && 
            [expr [llength [.can.c1 coords selected]]-1]<=7} {

            set messageString "\n\nA polygon cannot have less than three \
                coordinates."
# Call procedure to reconfigure message
            Set_Message $messageString
# Display this message for a period of time before resetting to the default bindings
            after 2500 {
# Set global "_CHOICE" to null and reset to the default bindings
                set _CHOICE ""
                Control ""
            }

# If the above is not the case, create coordinate points for the polygon, and call 
# the relevant procedures to set the correct bindings for the option.
        } else {
# Remove bounding-box and its points
            .can.c1 delete selpt boundingbox

# Set list to hold coordinates of item
            set coords [.can.c1 coords selected]

# The number of coordinates for a specific polygon will be required so
# coordinate points can be created.  Note that the first point is
# repeated at the end to close the polygon, so reduce the figure by 2
            set lengthCoords [expr ([llength $coords]-2)]

# For each x,y coordinate create a point	
            for {set a 0} {$a<$lengthCoords} {incr a 2} {
                set x [lindex $coords $a]
                set y [lindex $coords [expr ($a+1)]]
                eval ".can.c1 create rectangle $x $y $x $y -width 5 \
                    -outline black -tag copt"
            }
            if {$_CHOICE=="MOVE_POINT"} {
                Move_Poly_Point_BP1
                Drag_Poly_Point
                Move_Poly_Point_BR1
            }
            if {$_CHOICE=="ADD_POINT"} {
                Move_Poly_Point_BP1
                Drag_Poly_Point
                Add_Poly_Point_BR1
            }
            if {$_CHOICE=="DELETE_POINT"} {
                Delete_Poly_Point
            }
        }
    } else {
# Create the appropriate message and call procedure to display it
        if {$_CHOICE=="MOVE_POINT"} {
            set message_String "Use scaling facility for shapes

"
        }
        if {$_CHOICE=="ADD_POINT"} {
            set message_String "Points can only be added to polygon items

"
        }
        if {$_CHOICE=="DELETE_POINT"} {
            set message_String "Points can only be removed from polygon items

"
        }
# Call procedure to reconfigure message
        Set_Message $message_String
# Display the message for some time before resetting the default bindings
        after 2500 {Control ""}
    }
}

#=============================================================================#
###############################################################################
# This procedure creates the binding for <BP-1> for the Move and Add point    #
# options.  It identifies the point which has been selected and locates the   #
# surrounding points.                                                         #
###############################################################################
proc Move_Poly_Point_BP1 {} {

    bind .can.c1 <ButtonPress-1> {
        set Drag_motion 0

        set coords [.can.c1 coords selected]
        set lengthCoords [expr ([llength $coords]-2)]

# If BP1 on an entered point, this point is chosen
        .can.c1 addtag chosen closest \
            [.can.c1 canvasx %x] [.can.c1 canvasy %y] 1 "withtag copt"

# Set variables to hold the coordinates of the points on either side of 
# the selected one
        set prePt ""; set postPt ""

# Get a point-index for the chosen point, and split these into 
# x and y coordinates
        set pointCo [.can.c1 coords chosen]
        set pointCo_x [lindex $pointCo 0]
        set pointCo_y [lindex $pointCo 1]

# Search the list of coordinates for the shape to identify which point 
# has been chosen
        set index1 [lsearch $coords $pointCo_x]
        set index2 [lsearch $coords $pointCo_y]

# Variables used for dragging the point with cursor movement
        set current_x [.can.c1 canvasx %x]
        set current_y [.can.c1 canvasy %y]

# Lsearch locates the first item in the list with the pattern given.
# If there is a coordinate sharing either the same x or y value, this
# will be selected as matching.  So if the index values are not next
# to each other in the list, the following 'if' statement compares the
# corresponding coordinate value, locating which point has been chosen.
        if {[expr ($index2-1)]!=$index1} {
            if {$pointCo_x==[lindex $coords [expr ($index2-1)]]} {
                set index1 [expr ($index2-1)]
            } else {
                set index2 [expr ($index1+1)]
            }
        }

        if {$_CHOICE=="MOVE_POINT"} {
# If the chosen point is the first coordinate in the list, this sets
# the surrounding points to the 2nd and last points in list.
# Else it sets the surrounding points to the coordinates on either 
# side of the chosen point.
            if {$index1==0} {
                set prePt [list [lindex $coords 2] [lindex $coords 3]]
                set postPt [list [lindex $coords [expr ($lengthCoords-2)]] \
                    [lindex $coords [expr ($lengthCoords-1)]]]
            } else {
                set prePt [list [lindex $coords [expr ($index1-2)]] \
                    [lindex $coords [expr ($index1-1)]]]
                set postPt [list [lindex $coords [expr ($index1+2)]] \
                    [lindex $coords [expr ($index1+3)]]]
            }
        }
# If choice is ADDPT set first coordinate to the coordinates of that
# point, and the second to the point directly after it in the list.
# So the point we are adding is inserted between the selected point
# and the next point in coordinate list.
        if {$_CHOICE=="ADD_POINT"} {
            set prePt [list [lindex $coords [expr ($index2+1)]] \
                [lindex $coords [expr ($index2+2)]]]
            set postPt [list [lindex $coords $index1] [lindex $coords $index2]]
        }
    }
}

#=============================================================================#

###############################################################################
# This procedure drags the point across the canvas, and draws a connecting    #
# line to the points on either side.                                          #
###############################################################################
proc Drag_Poly_Point {} {

    bind .can.c1 <B1-Motion> {
# Delete the previous line
        .can.c1 delete molin
# Variable so redrawing only occurs if motion occurs
        if {$Drag_motion==0} {set Drag_motion 1}

# Calculate the difference in position after a cursor movement, and
# move the point the required amount
        set dx [expr ("%x"-$current_x)]
        set dy [expr ("%y"-$current_y)]

# Reset the current cursor position to the new position, so above
# calculation holds true for each movement
        set current_x [.can.c1 canvasx %x]
        set current_y [.can.c1 canvasy %y]

# Move command, and draw new connecting line
        .can.c1 move chosen $dx $dy
        eval ".can.c1 create line $prePt $current_x $current_y $postPt \
	    -fill $_COLOUR -tag molin"
    }
}

#=============================================================================#

###############################################################################
# This procedure creates the binding for <BR-1> for the Move-Point option.    #
# It locates the position to which the point is to be moved, amends the       #
# coordinates and redraws the shape.                                          #
###############################################################################
proc Move_Poly_Point_BR1 {} {

    bind .can.c1 <ButtonRelease-1> {
        set type [.can.c1 type selected]

# Delete the outline for shape
        .can.c1 delete lin molin
# Remove tag 'chosen' from point
        .can.c1 dtag chosen

# If type is polygon and first point in list is chosen, errors can
# occur with amending the list.  This removes the chance of error.
        if {$type=="polygon" || $type=="line"} {

# If the chosen point is the first coordinate in the list, then replace
# the 2nd and last coordinate points in the list.  Else replace the 
# points on either side of chosen point
            if {$index1==0} {
                set coords [lreplace $coords 0 1 $current_x $current_y]
                set coords [lreplace $coords $lengthCoords [expr ($lengthCoords+1)] \
                    $current_x $current_y]
            } else {
                set coords [lreplace $coords $index1 $index2 $current_x $current_y]
            }
# If type is other than polygon, set variable to hold all coordinates
# for item.
        } else {set coords [concat $coords $current_x $current_y]}

# Get a list of all tags for the altered item
        set itemIndex [lindex [.can.c1 gettags selected] 0]
        set attributes [lindex [.can.c1 gettags selected] 1]

# Set variable to hold value of items colour, taken from list of tags
        set itemColour [lindex $attributes 1]

        set itemWidth [lindex $attributes 2]

# Only if motion has occured, remove the shape and its points, and redraw
        if {$Drag_motion} {
            set Drag_motion 0
            .can.c1 delete selected copt
            if {$itemWidth==0} {
                eval ".can.c1 create $type $coords -fill $itemColour \
                    -tags $itemIndex"
            } else {
                eval ".can.c1 create $type $coords -fill $itemColour \
                    -width $itemWidth -tags $itemIndex"
            }
            .can.c1 addtag $attributes withtag $itemIndex

# Proc "Auto_Select" uses "Taglist in its full form, So reset.
            set taglist [.can.c1 gettags $itemIndex]
# Reselect the altered item
            Auto_Select $itemIndex
# Reset "_CHOICE" to null; And reimplement default bindings
            set _CHOICE ""
            Control ""
        }
    }
}

#=============================================================================#

###############################################################################
# This procedure creates the bindings for BR-1 when adding a point.  It       #
# locates the position that the new point is to be placed, draws it, amends   #
# the coordinate list and reselects the item with a new  b-box if required.   #
###############################################################################
proc Add_Poly_Point_BR1 {} {

    bind .can.c1 <ButtonRelease-1> {
# Identify the item type.  Point adjustment can only be carried out on polygons
        set type [.can.c1 type selected]

# Delete the outline for shape
        .can.c1 delete lin molin
# Remove tag 'chosen' from point
        .can.c1 dtag chosen

# Points can only be added to polygons. So only execute the following
# if type has been found to be a poly
        if {$type=="polygon" || $type=="line"} {

# If the chosen point is the first coordinate in the list, then replace
# the 2nd and last coordinate points in the list.  Else replace the 
# points on either side of chosen point
            if {$index1==0} {
                set coords [linsert $coords 2 $current_x $current_y]
            } else {
                set coords [linsert $coords [expr ($index2+1)] \
                    $current_x $current_y]
            }

# Get a list of all tags for the altered item
            set Taglist [.can.c1 gettags selected]
            set itemIndex [lindex $Taglist 0]
            set attributes [lindex $Taglist 1]

# Set variable to hold value of items colour, taken from list of tags
            set itemColour [lindex $attributes 1]
# Identify the width if the item is displayed in "Outline" style
            set itemWidth [lindex $attributes 2]

# Only if motion has occured, remove the shape, and redraw
            if {$Drag_motion} {
                set Drag_motion 0
# Delete the item that has been altered, and the 'coordinate points'
                .can.c1 delete selected copt

# Draw the altered item in the same style as it was
                if {$itemWidth==0} {
                    eval ".can.c1 create $type $coords -fill $itemColour \
                        -tags $itemIndex"
                } else {
                    eval ".can.c1 create $type $coords -width $itemWidth \
                        -fill $itemColour -tags $itemIndex"
                }
            }
        }
# Replace the second item in list_of_tags. 
# This holds the "attributes for the item
        .can.c1 addtag $attributes withtag $itemIndex

# Call procedure to automatically reselect the item
        Auto_Select $itemIndex
# Reset the global "_CHOICE"; And reimplement the default-bindings
        set _CHOICE ""
        Control ""
    }
}
